/*
 * IKA - IcedRobot Kiosk Application
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.icedrobot.ika.plugins.scm;

import java.io.File;
import java.io.IOException;
import org.icedrobot.ika.plugins.IkaPluginException;

/**
 * @author Mario Torre <neugens.limasoftware@gmail.com>
 */
public class GITRepository extends Repository {
    
    /**
     * Creates a new git Repository with the give name as ID, the
     * base directory as the location where the clone will be performed, the
     * relativePath as the name of the cloned repository, the remotePath as the
     * location of the source repository and branch as the version of the
     * repository to clone.
     */
    public GITRepository(String name, File baseDirectory, String relativePath,
                         String remotePath, String branch) {

        this.name = name;
        this.baseDirectory = baseDirectory;
        this.relativePath = relativePath;
        this.remotePath = remotePath;

        this.branch = branch;

        String canonicalPath;
        try {
             canonicalPath = baseDirectory.getCanonicalPath();
        } catch (IOException ex) {
            throw new IkaPluginException(ex.getMessage(), ex);
        }

        this.fullPath = new File(canonicalPath + File.separator + relativePath);
    }

    /**
     *
     */
    @Override
    public void makeClone() {
        String command = "git clone -b " + getBranch() + " " + remotePath;
        System.err.println(command);
        exec(command, getFullPath());
    }


    /**
     *
     */
    @Override
    public void create() {

        String command = "git init .";
        System.err.println(command);
        exec(command, getFullPath());

        command = "git add --all";
        System.err.println(command);
        exec(command, getFullPath());

        command = "git commit -m initial_repository --verbose";
        System.err.println(command);
        exec(command, getFullPath());

        command = "git branch " + branch;
        System.err.println(command);
        exec(command, getFullPath());

        command = "git checkout " + branch;
        System.err.println(command);
        exec(command, getFullPath());

        System.err.println(">>>>>>>> done: " + this + " <<<<<<<<");
    }

    /**
     *
     */
    @Override
    public Repository makeSubRepository(String name, String relativePath,
                                        String branch) {

        if (name.equals(".")) {
            name = this.getName();
        }
        Repository subRepository =
            new GITRepository(name, new File(getFullPath() + File.separator +
                                             getName()),
                              relativePath, null, branch);
        subRepository.create();
        return subRepository;
    }

    @Override
    public void applyPatchQueue() {
        String command = "guilt push --all";
        System.err.println(command);
        exec(command, getFullPath());
    }
}
